package de.fhdw.wtf.common.ast.type.test;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.AtomicType;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.CompositeType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.ThrownType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturn;
import de.fhdw.wtf.common.exception.ast.ASTException;
import de.fhdw.wtf.common.exception.parser.SumIsAnythingException;
import de.fhdw.wtf.common.token.DummyToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;
import de.fhdw.wtf.tooling.SyntaxCheck;

/**
 * Tests for Method standardizeElements in {@link de.fhdw.wtf.common.exception.parser.SumType}.
 */
public class StandardizeSumsTest extends TestCase {
	
	/**
	 * A > B und A > C. {A,B,C} => {A}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClass1() throws Exception {
		final String input = "Group:group=[Klasse:class={summe:{A,B,C};}; A:class={}; B:class=A+{}; C:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(1, standardizeElements.size());
		assertEquals(sum.getElements().iterator().next(), standardizeElements.iterator().next());
	}
	
	/**
	 * A > B, A > C und C > D. {A,B,C,D} => {A}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClass2() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{A,B,C,D};}; A:class={}; "
						+ "B:class=A+{}; C:class=A+{}; D:class=C+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(1, standardizeElements.size());
		assertEquals(sum.getElements().iterator().next(), standardizeElements.iterator().next());
	}
	
	/**
	 * A > B, A > C und C > D. {A,B,C,D,E} => {A,E}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClass3() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{A,B,C,D,E};}; A:class={};"
						+ " B:class=A+{}; C:class=A+{}; D:class=C+{};  E:class={};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
		assertEquals(sum.getElements().iterator().next(), standardizeElements.iterator().next());
	}
	
	/**
	 * D > B, D > C und B > A. {A,B,C,D} => {D}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClass4() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{A,B,C,D};}; A:class=B+{};"
						+ " B:class=D+{}; C:class=D+{}; D:class={};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(1, standardizeElements.size());
		assertEquals(sum.getElements().get(3), standardizeElements.iterator().next());
	}
	
	/**
	 * D > B, D > C und B > A. {B**,A,B,A*,C,D,D*} => {B**,D,A*,D*}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClassAndList1() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{B**,A,B,A*,C,D,D*};}; "
						+ "A:class=B+{}; B:class=D+{}; C:class=D+{}; D:class={};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(4, standardizeElements.size());
		final Iterator<Type> iterator = standardizeElements.iterator();
		// D
		assertEquals(sum.getElements().get(5), iterator.next());
		// A*
		assertEquals(sum.getElements().get(3), iterator.next());
		// D*
		assertEquals(sum.getElements().get(6), iterator.next());
		// B**
		assertEquals(sum.getElements().get(0), iterator.next());
	}
	
	/**
	 * A > B. {A*,B*} => {A*,B*}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testList1() throws Exception {
		final String input = "Group:group=[Klasse:class={summe:{A*,B*};}; A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
	}
	
	/**
	 * A > B und A > C. {{A,B},C} => {A}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testSum1() throws Exception {
		final String input = "Group:group=[Klasse:class={summe:{{A,B},C};}; A:class={}; B:class=A+{}; C:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(1, standardizeElements.size());
		assertEquals(
				((SumType) sum.getElements().iterator().next()).getElements().iterator().next(),
				standardizeElements.iterator().next());
	}
	
	/**
	 * A > B und A > C. {{A,B,{A*,B*}},C,C*} => {A,A*,B*,C*}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testSum2() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{{A,B,{A*,B*}},C,C*};}; A:class={};"
						+ " B:class=A+{}; C:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(4, standardizeElements.size());
		
		final Iterator<Type> iterator1 = standardizeElements.iterator();
		final Iterator<Type> iterator0 = sum.getElements().iterator();
		final Iterator<Type> iterator2 = ((SumType) iterator0.next()).getElements().iterator();
		
		final Type a = iterator2.next();
		iterator2.next();
		final Iterator<Type> iterator3 = ((SumType) iterator2.next()).getElements().iterator();
		final Type listA = iterator3.next();
		final Type listB = iterator3.next();
		iterator0.next();
		final Type listC = iterator0.next();
		
		// A
		assertEquals(a, iterator1.next());
		// A*
		assertEquals(listA, iterator1.next());
		// B*
		assertEquals(listB, iterator1.next());
		// C*
		assertEquals(listC, iterator1.next());
	}
	
	/**
	 * A > B und A > C. {A,B,C,C*,A*,{B*,A*},(name:String,alter:Integer),[String -> Integer]} => {A,A*,B*,C*,[String ->
	 * Integer],(name:String,alter:Integer)}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testClassProductMapListSum1() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe:{A,B,C,C*,A*,{B*,A*},"
						+ "(name:String,alter:Integer),[String -> Integer]};}; A:class={}; B:class=A+{}; C:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(6, standardizeElements.size());
		// A
		assertEquals(sum.getElements().get(0), standardizeElements.get(0));
		// A*
		assertEquals(sum.getElements().get(4), standardizeElements.get(1));
		// B*
		assertEquals(((SumType) sum.getElements().get(5)).getElements().get(0), standardizeElements.get(2));
		// C*
		assertEquals(sum.getElements().get(3), standardizeElements.get(3));
		// [String -> Integer]
		assertEquals(sum.getElements().get(7), standardizeElements.get(4));
		// (name:String,alter:Integer)
		assertEquals(sum.getElements().get(6), standardizeElements.get(5));
	}
	
	/**
	 * {[String->Integer],[String->Integer]} => {[String->Integer]}.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testMap1() throws Exception {
		final String input = "Group:group=[Klasse:class={summe:{[String -> Integer],[String -> Integer]};};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(1, standardizeElements.size());
		assertEquals(sum.getElements().get(0), standardizeElements.iterator().next());
	}
	
	/**
	 * A > B. {[A->Integer],[B->Integer]} => {[A->Integer],[B->Integer]}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testMap2() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe: {[A->Integer],[B->Integer]};}; " + "A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
	}
	
	/**
	 * A > B. {[Integer->A],[Integer->B]} => {[Integer->A],[Integer->B]}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testMap3() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe: {[Integer->A],[Integer->B]};}; " + "A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
	}
	
	/**
	 * A > B. {[B->A],[A->B]} => {[B->A],[A->B]}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testMap4() throws Exception {
		final String input = "Group:group=[Klasse:class={summe: {[B->A],[A->B]};}; A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
	}
	
	/**
	 * A > B. {[B->A],[A->Integer]} => {[B->A],[A->Integer]}
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testMap5() throws Exception {
		final String input = "Group:group=[Klasse:class={summe: {[B->A],[A->Integer]};}; A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		
		final List<Type> standardizeElements = sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		assertEquals(2, standardizeElements.size());
	}
	
	/**
	 * A > B. {Anything,A,B} => Summe "Anything"
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testAnything1() throws Exception {
		final String input = "Group:group=[Klasse:class={summe: {Anything,A,B};}; A:class={}; B:class=A+{};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		try {
			sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
				
				@Override
				public List<Type> handle(final AtomicType atomicType) {
					return new ArrayList<>();
				}
				
				@Override
				public List<Type> handle(final CompositeType compositeType) {
					return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
						
						@Override
						public List<Type> handle(final ListType list) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final MapType map) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final ProductType product) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final SumType sum2) {
							return sum2.getElements();
						}
						
						@Override
						public List<Type> handle(final ThrownType thrownType) {
							return new ArrayList<>();
						}
					});
				}
				
				@Override
				public List<Type> handle(final TypeProxy typeProxy) {
					return new ArrayList<>();
				}
			});
			
			fail();
		} catch (final SumIsAnythingException e) {
			// Nothing
		}
	}
	
	/**
	 * A > B. {{Anything,C},A,B} => Summe "Anything"
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.SumIsAnythingException},
	 *             {@link de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testAnything2() throws Exception {
		final String input =
				"Group:group=[Klasse:class={summe: {{Anything,C},A,B};};" + " A:class={}; B:class=A+{}; C:class={};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		final Group group = model.getGroups().iterator().next();
		final ClassType klasse = (ClassType) group.getGroupElements().iterator().next();
		final SumType sum = (SumType) klasse.getAttributes().iterator().next().getAttrType();
		try {
			sum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
				
				@Override
				public List<Type> handle(final AtomicType atomicType) {
					return new ArrayList<>();
				}
				
				@Override
				public List<Type> handle(final CompositeType compositeType) {
					return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
						
						@Override
						public List<Type> handle(final ListType list) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final MapType map) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final ProductType product) {
							return new ArrayList<>();
						}
						
						@Override
						public List<Type> handle(final SumType sum2) {
							return sum2.getElements();
						}
						
						@Override
						public List<Type> handle(final ThrownType thrownType) {
							return new ArrayList<>();
						}
					});
				}
				
				@Override
				public List<Type> handle(final TypeProxy typeProxy) {
					return new ArrayList<>();
				}
			});
			fail();
		} catch (final SumIsAnythingException e) {
			// Nothing
		}
	}
	
	/**
	 * {A!}! => Result: A!.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsThrownType() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ThrownType actualThrownType =
				ThrownType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		actualSum.add(actualThrownType);
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final ThrownType expectedThrownType =
				ThrownType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		// Test
		assertEquals(standardizedType, expectedThrownType);
	}
	
	/**
	 * {A!} => Result: A!.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testSumContainsThrownType() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ThrownType actualThrownType =
				ThrownType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		actualSum.add(actualThrownType);
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final ThrownType expectedThrownType =
				ThrownType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		// Test
		assertEquals(1, standardizeElements.size());
		final Type first = standardizeElements.get(0);
		assertEquals(first, expectedThrownType);
	}
	
	/**
	 * {A}! => Result: A!.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsNormalType() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final ThrownType expectedThrownType =
				ThrownType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		// Test
		assertEquals(standardizedType, expectedThrownType);
	}
	
	/**
	 * {A,B}! => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsNormalTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(2, sum.getElements().size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {A!,B}! => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsOneThrownTypeFirstAndOneNormalThrownTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(2, sum.getElements().size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {A,B!}! => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsOneThrownTypeSecondAndOneNormalThrownTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(2, sum.getElements().size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {B!,A}! => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsOneThrownTypeSecondAndOneNormalThrownTypesAndDifferentOrder()
			throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(2, sum.getElements().size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {A!,B!}! => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainsThrownTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(2, sum.getElements().size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {A!,B} => Result: {B,A!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testSumContainsOneThrownTypeFirst() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		
		// Test
		assertEquals(2, standardizeElements.size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizeElements, expectedSum.getElements());
	}
	
	/**
	 * {A,B!} => Result: {A,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testSumContainsOneThrownTypeSecond() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertEquals(2, standardizeElements.size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizeElements, expectedSum.getElements());
	}
	
	/**
	 * {A!,B!} => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testSumContainsTwoThrownTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertEquals(2, standardizeElements.size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizeElements, expectedSum.getElements());
	}
	
	/**
	 * {A!,B!,A!} => Result: {A!,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testDoubleThrownElements() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertEquals(2, standardizeElements.size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizeElements, expectedSum.getElements());
		
	}
	
	/**
	 * {A!,B!,C}! => Result: {A!,B!,C!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingThrownTypesAndNormalType() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		final IdentifierToken c = IdentifierToken.create("C", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(c))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type standardizedType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(c)))));
		
		// Test
		assertTrue(standardizedType instanceof SumType);
		final SumType sum = (SumType) standardizedType;
		assertEquals(3, sum.getElements().size());
		assertEquals(3, expectedSum.getElements().size());
		this.assertEqualsOnlists(sum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {(x:String, y:Integer)}! => Result: Error.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingProductType() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken string = IdentifierToken.create("String", dummyPosition);
		final IdentifierToken integer = IdentifierToken.create("Integer", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ProductType actualProduct = ProductType.create(dummyToken);
		actualProduct.addElement(ProductElementType.create(
				"x",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(string))),
				dummyToken));
		actualProduct.addElement(ProductElementType.create(
				"y",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(integer))),
				dummyToken));
		actualSum.add(actualProduct);
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		try {
			thrownType.standardize();
			
			assertFalse(true);
		} catch (final ASTException e) {
			assertTrue(true);
			assertEquals("A product can not be thrown.", e.getMessage());
		}
	}
	
	/**
	 * {(x:String, y:Integer),(a:Integer,b:String)}! => Result: Error.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingProductTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken string = IdentifierToken.create("String", dummyPosition);
		final IdentifierToken integer = IdentifierToken.create("Integer", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ProductType actualFirstProduct = ProductType.create(dummyToken);
		actualFirstProduct.addElement(ProductElementType.create(
				"x",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(string))),
				dummyToken));
		actualFirstProduct.addElement(ProductElementType.create(
				"y",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(integer))),
				dummyToken));
		actualSum.add(actualFirstProduct);
		final ProductType actualSecondProduct = ProductType.create(dummyToken);
		actualSecondProduct.addElement(ProductElementType.create(
				"a",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(integer))),
				dummyToken));
		actualSecondProduct.addElement(ProductElementType.create(
				"b",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(string))),
				dummyToken));
		actualSum.add(actualSecondProduct);
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		try {
			thrownType.standardize();
			
			assertFalse(true);
		} catch (final ASTException e) {
			assertTrue(true);
			assertEquals("A product can not be thrown.", e.getMessage());
		}
	}
	
	/**
	 * {C*}! => Result: Error.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingList() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken c = IdentifierToken.create("C", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ListType actualList =
				ListType.create(dummyToken, TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(c))));
		actualSum.add(actualList);
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		try {
			thrownType.standardize();
			
			assertFalse(true);
		} catch (final ASTException e) {
			assertTrue(true);
			assertEquals("A list can not be thrown.", e.getMessage());
		}
	}
	
	/**
	 * {C*, A*}! => Result: Error.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingLists() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken c = IdentifierToken.create("C", dummyPosition);
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		final ListType actualFirstList =
				ListType.create(dummyToken, TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(c))));
		actualSum.add(actualFirstList);
		final ListType actualSecondList =
				ListType.create(dummyToken, TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		actualSum.add(actualSecondList);
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		try {
			thrownType.standardize();
			
			assertFalse(true);
		} catch (final ASTException e) {
			assertTrue(true);
			assertEquals("A list can not be thrown.", e.getMessage());
		}
	}
	
	/**
	 * {B!,B,B,B!} => Result: {B,B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testSumTypesManyTimes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final List<Type> standardizeElements = actualSum.standardize().accept(new TypeVisitorReturn<List<Type>>() {
			
			@Override
			public List<Type> handle(final AtomicType atomicType) {
				return new ArrayList<>();
			}
			
			@Override
			public List<Type> handle(final CompositeType compositeType) {
				return compositeType.accept(new CompositeTypeVisitorReturn<List<Type>>() {
					
					@Override
					public List<Type> handle(final ListType list) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final MapType map) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final ProductType product) {
						return new ArrayList<>();
					}
					
					@Override
					public List<Type> handle(final SumType sum2) {
						return sum2.getElements();
					}
					
					@Override
					public List<Type> handle(final ThrownType thrownType) {
						return new ArrayList<>();
					}
				});
			}
			
			@Override
			public List<Type> handle(final TypeProxy typeProxy) {
				return new ArrayList<>();
			}
		});
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertEquals(2, standardizeElements.size());
		assertEquals(2, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizeElements, expectedSum.getElements());
	}
	
	/**
	 * {B!,B,B,B!}! => Result: {B!}.
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumTypesManyTimes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		final Type actualType = thrownType.standardize();
		
		// Prepare Expected
		final SumType expectedSum = SumType.create(dummyToken);
		expectedSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		// Test
		assertTrue(actualType instanceof SumType);
		final SumType standardizedSum = (SumType) actualType;
		assertEquals(1, standardizedSum.getElements().size());
		assertEquals(1, expectedSum.getElements().size());
		this.assertEqualsOnlists(standardizedSum.getElements(), expectedSum.getElements());
	}
	
	/**
	 * {A*,(x:String,b:String),B!,B,B,B!,[A->Integer]}! => Result: Error
	 * 
	 * @throws SumIsAnythingException
	 *             If it contains Anything throw specified Exception
	 */
	@Test
	public void testThrownSumContainingDifferentTypes() throws SumIsAnythingException {
		
		final Token dummyToken = DummyToken.getInstance();
		final Position dummyPosition = DummyToken.getDummyPosition();
		final IdentifierToken c = IdentifierToken.create("C", dummyPosition);
		final IdentifierToken a = IdentifierToken.create("A", dummyPosition);
		final IdentifierToken string = IdentifierToken.create("String", dummyPosition);
		final IdentifierToken integer = IdentifierToken.create("Integer", dummyPosition);
		final IdentifierToken b = IdentifierToken.create("B", dummyPosition);
		
		// Prepare Actual
		final SumType actualSum = SumType.create(dummyToken);
		
		final ListType actualFirstList =
				ListType.create(dummyToken, TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))));
		actualSum.add(actualFirstList);
		
		final ProductType actualSecondProduct = ProductType.create(dummyToken);
		actualSecondProduct.addElement(ProductElementType.create(
				"x",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(string))),
				dummyToken));
		actualSecondProduct.addElement(ProductElementType.create(
				"b",
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(string))),
				dummyToken));
		actualSum.add(actualSecondProduct);
		
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b))));
		actualSum.add(ThrownType.create(
				dummyToken,
				TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(b)))));
		
		final MapType actualFirstMap =
				MapType.create(
						dummyToken,
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(a))),
						TypeProxy.create(dummyToken, ByNameState.create(UnqualifiedName.create(integer))));
		actualSum.add(actualFirstMap);
		
		final ThrownType thrownType = ThrownType.create(DummyToken.getInstance(), actualSum, DummyToken.getInstance());
		
		try {
			thrownType.standardize();
			
			assertFalse(true);
		} catch (final ASTException e) {
			assertTrue(true);
			assertEquals("A list can not be thrown.", e.getMessage());
		}
	}
	
	/**
	 * Check if the two Lists of types are the same.
	 * 
	 * @param expected
	 *            the first List with the elements, who are expected.
	 * @param actual
	 *            the list with the elements, who are actual there.
	 */
	private void assertEqualsOnlists(final List<Type> expected, final List<Type> actual) {
		
		assertEquals(expected.size(), actual.size());
		
		for (int i = 0; i < expected.size(); i++) {
			final Type currentExpected = expected.get(i);
			final Type currentActual = actual.get(i);
			
			assertEquals(currentExpected, currentActual);
		}
	}
}
