package de.fhdw.wtf.tooling.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.net.URI;
import java.util.Collection;
import java.util.Iterator;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.visitor.GroupElementVisitor;
import de.fhdw.wtf.common.exception.editor.CheckException;
import de.fhdw.wtf.common.exception.editor.MarkableCheckException;
import de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions;
import de.fhdw.wtf.common.exception.parser.NoColonException;
import de.fhdw.wtf.common.exception.parser.NoPlusSymbolException;
import de.fhdw.wtf.common.exception.referencer.InvalidTypeReferenceException;
import de.fhdw.wtf.common.exception.walker.AbstractOperationsException;
import de.fhdw.wtf.common.exception.walker.BaseTypeInheritanceException;
import de.fhdw.wtf.common.exception.walker.DoubleGroupcomponentException;
import de.fhdw.wtf.testutil.TestUtil;
import de.fhdw.wtf.tooling.SyntaxCheck;

/**
 * Test {@link SyntaxCheck} - scanning, parsing and referencing and walker checks - in one integration test.
 */
public class SyntaxCheckTest {
	
	/**
	 * Path to the file directory with the test resources.
	 */
	private static final String PATH_TO_TEST_FILE_DIR = "src/test/java/de/fhdw/wtf/tooling/test/files/";
	
	/**
	 * Suffix for model files.
	 */
	private static final String MODEL_SUFFIX = "model";
	
	/**
	 * Tests the scanning, parsing, referencing and checks of a model-string.
	 * 
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testSimpleModelFromString() throws Exception {
		final String input = TestUtil.readFileToString(PATH_TO_TEST_FILE_DIR + "Person" + '.' + MODEL_SUFFIX);
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		final Model model = instance.getModelFromString(input);
		
		// -- should be one group --//
		final Collection<Group> groups = model.getGroups();
		assertEquals(1, groups.size());
		
		// -- should be three groupElements --//
		final Group group1 = groups.iterator().next();
		final Collection<GroupElement> groupElements = group1.getGroupElements();
		assertEquals(3, groupElements.size());
		final Iterator<GroupElement> iterator = groupElements.iterator();
		
		// -- first groupElement should have one attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(1, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(1, clss.getModifiers().size());
			}
		});
		
		// -- second groupElement should have no attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(0, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(0, clss.getModifiers().size());
			}
		});
		
		// -- third groupElement should have one attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(1, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(0, clss.getModifiers().size());
			}
		});
	}
	
	/**
	 * Tests the scanning, parsing, referencing and checks of a model-file.
	 * 
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testSimpleModelFromFile() throws Exception {
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		final URI uri = new URI(PATH_TO_TEST_FILE_DIR + "Person" + '.' + MODEL_SUFFIX);
		final Model model = instance.getModelFromFile(uri);
		
		// -- should be one group --//
		final Collection<Group> groups = model.getGroups();
		assertEquals(1, groups.size());
		
		// -- should be three groupElements --//
		final Group group1 = groups.iterator().next();
		final Collection<GroupElement> groupElements = group1.getGroupElements();
		assertEquals(3, groupElements.size());
		final Iterator<GroupElement> iterator = groupElements.iterator();
		
		// -- first groupElement should have one attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(1, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(1, clss.getModifiers().size());
			}
		});
		
		// -- second groupElement should have no attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(0, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(0, clss.getModifiers().size());
			}
		});
		
		// -- third groupElement should have one attribute --//
		iterator.next().accept(new GroupElementVisitor() {
			
			@Override
			public void handle(final Group group) {
				fail();
			}
			
			@Override
			public void handle(final ClassType clss) {
				assertEquals(1, clss.getAttributes().size());
				assertEquals(0, clss.getOperations().size());
				assertEquals(0, clss.getModifiers().size());
			}
		});
	}
	
	/**
	 * Tests the scanning, parsing, referencing and checks of a model-file with some parser exceptions.
	 * 
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testInvalidModelFileWithParserExceptions() throws Exception {
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		final URI uri = new URI(PATH_TO_TEST_FILE_DIR + "InvalidPerson" + '.' + MODEL_SUFFIX);
		try {
			instance.getModelFromFile(uri);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertEquals(2, e.getExeptionCount());
			final Iterator<CheckException> iterator = e.iterator();
			final MarkableCheckException noColonException = (MarkableCheckException) iterator.next();
			assertEquals(NoColonException.class, noColonException.getCause().getClass());
			final MarkableCheckException noSemicolonException = (MarkableCheckException) iterator.next();
			assertEquals(NoPlusSymbolException.class, noSemicolonException.getCause().getClass());
		}
	}
	
	/**
	 * Tests the scanning, parsing, referencing and checks of a model-file with some walker exceptions.
	 * 
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testInvalidModelFileWithWalkerExceptions() throws Exception {
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		final URI uri = new URI(PATH_TO_TEST_FILE_DIR + "ProblemModel" + '.' + MODEL_SUFFIX);
		try {
			instance.getModelFromFile(uri);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertEquals(2, e.getExeptionCount());
			final Iterator<CheckException> iterator = e.iterator();
			assertEquals(DoubleGroupcomponentException.class, iterator.next().getCause().getClass());
			assertEquals(AbstractOperationsException.class, iterator.next().getCause().getClass());
		}
	}
	
	/**
	 * Tests the scanning, parsing, referencing and checks of a model-file with a referencer exceptions.
	 * 
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testInvalidModelFileWithReferencerException() throws Exception {
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		final URI uri = new URI(PATH_TO_TEST_FILE_DIR + "ProblemModel2" + '.' + MODEL_SUFFIX);
		try {
			instance.getModelFromFile(uri);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertEquals(1, e.getExeptionCount());
			final Iterator<CheckException> iterator = e.iterator();
			assertEquals(InvalidTypeReferenceException.class, iterator.next().getCause().getClass());
		}
	}
	
	/**
	 * @throws Exception
	 *             {@link java.io.FileNotFoundException}, {@link java.io.IOException}, {@link MultipleCheckExceptions},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicDependencyException}, {@link InterruptedException},
	 *             {@link de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException},
	 *             {@link java.util.concurrent.ExecutionException}
	 */
	@Test
	public void testInheritanceFromBaseType() throws Exception {
		final String input = "personGroup:group=[Person:class=String + {name:String*;};];";
		final SyntaxCheck instance = SyntaxCheck.getInstance();
		
		try {
			instance.getModelFromString(input);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertEquals(1, e.getExeptionCount());
			final Iterator<CheckException> iterator = e.iterator();
			assertEquals(BaseTypeInheritanceException.class, iterator.next().getCause().getClass());
		}
	}
}
